/**
 * HibernatePersistenceDaoManager.java
 *
 * Created on 31 Jul 2010 23:16:26
 */
package org.blueoxygen.cimande.persistence.hibernate.dao;

import java.sql.Timestamp;
import java.util.List;

import org.blueoxygen.cimande.commons.Orderby;
import org.blueoxygen.cimande.persistence.GenericPersistenceManager;
import org.blueoxygen.cimande.persistence.dao.PersistenceDaoManager;
import org.blueoxygen.cimande.security.SessionCredentials;
import org.blueoxygen.cimande.security.User;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author Dian Aditya
 * 
 */
@Service
public class HibernatePersistenceDaoManager<T> implements
		PersistenceDaoManager<T> {

	@Autowired
	private SessionCredentials credentials;

	@Autowired
	protected SessionFactory sessionFactory;

	@Autowired
	protected GenericPersistenceManager manager;

	protected Orderby ob = new Orderby();

	public List<T> findAll(Class<T> type) {
		return findAllSorted(type, null);
	}

	public List<T> findAllSorted(Class<T> type, String sortProperty) {
		return findAllSortedDirected(type, sortProperty, Direction.ASC);
	}

	@SuppressWarnings("unchecked")
	public List<T> findAllSortedDirected(Class<T> type, String sortField,
			Direction direction) {

		sortField = sortField == null ? "" : sortField;

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				type);

		if (!sortField.trim().equalsIgnoreCase("")) {
			if (direction.equals(Direction.ASC)) {
				criteria.addOrder(Order.asc(sortField));
			} else if (direction.equals(Direction.DESC)) {
				criteria.addOrder(Order.desc(sortField));
			} else {
				criteria.addOrder(Order.asc(sortField));
			}
		}

		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	public List<T> find(Class<T> type, String field, String contain) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				type);
		criteria.add(Restrictions.like(field, contain, MatchMode.ANYWHERE));

		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	public T getById(Class<T> type, String id) {
		return (T) sessionFactory.getCurrentSession().get(type, id);
	}

	@SuppressWarnings("unchecked")
	public T getByUniqueField(Class<T> type, String identifier, String fieldName) {
		String hql = "SELECT o FROM " + type.getName() + " o WHERE o."
				+ fieldName + " =:identifier";
		Query query = sessionFactory.getCurrentSession().createQuery(hql)
				.setParameter("identifier", identifier);

		return (T) query.list();
	}

	public void remove(T objectToRemove) {
		sessionFactory.getCurrentSession().delete(objectToRemove);
	}

	public void merge(T object) {
		sessionFactory.getCurrentSession().merge(object);
	}

	public void persist(T object) {
		sessionFactory.getCurrentSession().saveOrUpdate(object);
	}

	public Timestamp getCurretTime() {
		return new Timestamp(System.currentTimeMillis());
	}

	public User getCurrentUser() {
		return credentials.getCurrentUser();
	}

}
