/**
 * DefaultContentHandlerManager.java
 *
 * Created on 6 Sep 2010 15:30:18
 */
package org.blueoxygen.cimande.rest.manager;

import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletResponse;

import org.apache.struts2.ServletActionContext;
import org.blueoxygen.cimande.rest.handler.ContentHandler;

import com.opensymphony.xwork2.inject.Container;
import com.opensymphony.xwork2.inject.Inject;

/**
 * @author Dian Aditya
 *
 */
public class DefaultContentHandlerManager implements ContentHandlerManager {

	private Map<String, ContentHandler> handlerByExtension = new HashMap<String, ContentHandler>();
	private Map<String, ContentHandler> handlerByContentType = new HashMap<String, ContentHandler>();

	@Inject
	public void setContainer(Container container) {
		Set<String> handlerNames = container
				.getInstanceNames(ContentHandler.class);

		for (String name : handlerNames) {
			ContentHandler handler = container.getInstance(
					ContentHandler.class, name);
			if (handler.getExtension() != null)
				handlerByExtension.put(handler.getExtension(), handler);
			if (handler.getContentType() != null)
				handlerByContentType.put(handler.getContentType(), handler);
		}
	}

	public Map<String, ContentHandler> getHandlerByExtension() {
		return handlerByExtension;
	}

	public Map<String, ContentHandler> getHandlerByContentType() {
		return handlerByContentType;
	}

	public void writeObject(ContentHandler handler, Object target)
			throws IOException {
		HttpServletResponse response = ServletActionContext.getResponse();
		StringWriter writer = new StringWriter();
		handler.fromObject(target, writer);
		String text = writer.toString();
		if (text.length() > 0) {
			byte[] data = text.getBytes("UTF-8");
			response.setContentLength(data.length);
			response.setContentType(handler.getContentType());
			response.getOutputStream().write(data);
			response.getOutputStream().close();
		}
	}

}
