package org.blueoxygen.cimande.rolesite.service;

import java.util.List;

import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.blueoxygen.cimande.role.RoleSite;
import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/*******************************
 * @author Praditya Eldorado 
 *         aldo.donal@gmail.com
 *******************************/

@Repository
@Transactional(readOnly = true)
public class ServiceRoleSiteImpl extends
		HibernatePersistenceDaoManager<RoleSite> implements ServiceRoleSite {
	@Transactional
	public void save(RoleSite rolesite) {
		if (rolesite != null) {
			if (rolesite.getId() == null
					|| "".equalsIgnoreCase(rolesite.getId())) {
				rolesite.getLogInformation().setCreateBy(
						getCurrentUser().getId());
				rolesite.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				rolesite.getLogInformation().setCreateDate(getCurretTime());
				rolesite.getLogInformation().setLastUpdateDate(getCurretTime());

				rolesite.getLogInformation().setActiveFlag(1);
				persist(rolesite);
			} else {
				rolesite.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				rolesite.getLogInformation().setLastUpdateDate(getCurretTime());

				merge(rolesite);
			}

		}

	}

	@Transactional
	public void delete(RoleSite rolesite) {
		if (rolesite != null && rolesite.getId() != null) {
			rolesite = getById(RoleSite.class, rolesite.getId());

			remove(rolesite);
		}
	}

	@Transactional(readOnly = true)
	public RoleSite getRoleSiteById(String id) {
		return getById(RoleSite.class, id);
	}

	@Transactional(readOnly = true)
	public List<RoleSite> getAllRoleSites(String orderby) {
		return findAllSorted(RoleSite.class, orderby);
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<RoleSite> findRoleSitesByField(String field, String value) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RoleSite.class);
		criteria.add(Restrictions.like(field, value));

		return criteria.list();
	}

	@Transactional(readOnly = true)
	public int getRowCountByCriteria(String role, String site) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RoleSite.class);

		criteria.add(Restrictions
				.like("role.id", role + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions
				.like("site.id", site + "", MatchMode.ANYWHERE));
		criteria.setProjection(Projections.rowCount());

		return new Integer(criteria.uniqueResult() + "");
	}

	@Transactional(readOnly = true)
	public List<RoleSite> findRoleSiteByRoleId(String roleId) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RoleSite.class);
		criteria.add(Restrictions.like("role.id", roleId));
		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<RoleSite> findRoleSites(String role, String site,
			String orderby, int maxResults, int firstResult) {
		Criteria crit = sessionFactory.getCurrentSession().createCriteria(
				RoleSite.class);
		crit.add(Restrictions.like("role.id", role));
		crit.add(Restrictions.like("site.id", site));
		crit.setMaxResults(maxResults);
		crit.setFirstResult(firstResult);
		crit.addOrder(Order.asc(orderby));
		return crit.list();
	}

	@Transactional(readOnly = true)
	public List<RoleSite> getList(String roleId, int activeFlag) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RoleSite.class);
		activeFlag = 1;
		criteria.add(Restrictions.like("role.id", roleId));
		criteria.add(Restrictions.like("logInformation.activeFlag", activeFlag));
		return criteria.list();
	}

	@Transactional(readOnly = true)
	public List<RoleSite> getLists(String parentId, String roleId,
			int activeflag) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				RoleSite.class);
		activeflag = 1;
		criteria.add(Restrictions.or(Restrictions.like("role.id", roleId),
				Restrictions.like("role.id", parentId)));
		criteria.add(Restrictions.like("logInformation.activeFlag", activeflag));
		return criteria.list();
	}
}
