/**
 * LoginInterceptor.java
 *
 * Created on 6 Sep 2010 21:08:11
 */
package org.blueoxygen.cimande.security.login.interceptors;

import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.struts2.ServletActionContext;
import org.blueoxygen.cimande.rest.manager.ContentHandlerManager;

import com.opensymphony.xwork2.Action;
import com.opensymphony.xwork2.ActionInvocation;
import com.opensymphony.xwork2.inject.Inject;
import com.opensymphony.xwork2.interceptor.Interceptor;
import com.opensymphony.xwork2.util.TextParseUtil;

/**
 * @author Dian Aditya
 * 
 */
public class LoginInterceptor implements Interceptor {

	public static String LOGIN_KEY = "loggedIn";
	public static String LOGIN_USER = "user";
	public static String LOGIN_CIMANDE_USER = "GA_USER";
	public static String LOGIN_CIMANDE_SITE = "GS_USER";
	public static String CIMANDE_USER = "CIMANDE_USER";
	public static String CIMANDE_SITE = "CIMANDE_SITE";

	protected Set<String> extensions;
	protected Set<String> filterUrls;

	protected String loginAction = "/backend/user/index";
	protected String loginPage = "/backend/user/index";
	protected String logoutAction = "/backend/user/logout";

	protected String redirectUri;

	protected ContentHandlerManager handlerManager;

	@Inject
	public void setContentHandlerManager(ContentHandlerManager handlerManager) {
		this.handlerManager = handlerManager;
	}

	public void setLoginAction(String loginAction) {
		this.loginAction = loginAction;
	}

	public void setLoginPage(String loginPage) {
		this.loginPage = loginPage;
	}

	public void setLogoutAction(String logoutAction) {
		this.logoutAction = logoutAction;
	}

	public void setIgnoreExtensions(String ignoreExtension) {
		this.extensions = TextParseUtil
				.commaDelimitedStringToSet(ignoreExtension);
		System.out.println(extensions);
	}

	public void setFilterUrl(String urls) {
		this.filterUrls = TextParseUtil.commaDelimitedStringToSet(urls);
	}

	public void destroy() {
	}

	public void init() {

	}

	public String intercept(ActionInvocation invocation) throws Exception {
		HttpServletRequest request = ServletActionContext.getRequest();
		HttpSession session = request.getSession(true);
		String servletPath = request.getServletPath();
		String extension = servletPath.substring(
				servletPath.lastIndexOf('.') + 1).toLowerCase();

		boolean intercept = false;

		for (String url : filterUrls) {
			if (request.getServletPath().startsWith(url)) {
				intercept = true;

				break;
			}
		}

		System.out.println(intercept);

		if (!intercept) {
			return invocation.invokeActionOnly();
		}

		if (servletPath.equals(loginAction) || servletPath.equals(loginPage)
				|| servletPath.equals(logoutAction)
				|| extensions.contains(extension)) {

			return invocation.invokeActionOnly();
		} else if (session.getAttribute(LOGIN_CIMANDE_USER) == null) {
			if (handlerManager.getHandlerByExtension().containsKey(
					servletPath.substring(servletPath.lastIndexOf('/') + 1))) {

				return null;
			}

			session.setAttribute("REDIRECT_URI", servletPath.substring(1));

			return Action.LOGIN;
		}

		return invocation.invoke();
	}

	public String getRedirectUri() {
		return redirectUri;
	}

}
