package org.blueoxygen.cimande.site.service;

import java.util.List;

import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.blueoxygen.cimande.site.Site;
import org.hibernate.Criteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/*******************************
 * @author Praditya Eldorado 
 *         aldo.donal@gmail.com
 *******************************/

@Repository
@Transactional(readOnly = true)
public class SiteServiceImpl extends HibernatePersistenceDaoManager<Site>
		implements SiteService {

	@Transactional
	public void save(Site site) {
		if (site != null) {
			if (site.getId() == null || "".equalsIgnoreCase(site.getId())) {
				site.getLogInformation().setCreateBy(getCurrentUser().getId());
				site.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				site.getLogInformation().setCreateDate(getCurretTime());
				site.getLogInformation().setLastUpdateDate(getCurretTime());
				site.getLogInformation().setActiveFlag(1);
				persist(site);
			} else {
				site.getLogInformation().setLastUpdateBy(
						getCurrentUser().getId());
				site.getLogInformation().setLastUpdateDate(getCurretTime());

				merge(site);
			}

		}

	}

	@Transactional
	public void delete(Site site) {
		if (site != null && site.getId() != null) {
			site = getById(Site.class, site.getId());

			remove(site);
		}
	}

	@Transactional(readOnly = true)
	public Site getSiteById(String id) {
		return getById(Site.class, id);
	}

	@Transactional(readOnly = true)
	public List<Site> getAllSites(String orderbyName) {
		return findAllSorted(Site.class, orderbyName);
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<Site> findSites(String name, String description, String title,
			String url_branding, String site_url, String admin_email,
			int notify_flag, String notify_email, String notify_from,
			String notify_message, int activeFlag, String workspace_type,
			String orderby, int maxResults, int firstResult) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Site.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("title", title + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("url_branding", url_branding + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("site_url", site_url + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("admin_email", admin_email + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("notify_email", notify_email + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("notify_from", notify_from + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("notify_message", notify_message + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("workspace_type", workspace_type + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.eq("notify_flag", notify_flag));
		criteria.add(Restrictions.eq("logInformation.activeFlag", activeFlag));
		criteria.addOrder(Order.asc(orderby));
		criteria.setMaxResults(maxResults);
		criteria.setFirstResult(firstResult);

		return criteria.list();
	}

	@SuppressWarnings("unchecked")
	@Transactional(readOnly = true)
	public List<Site> findSitesByField(String field, String value) {
		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Site.class);
		criteria.add(Restrictions.like(field, value));

		return criteria.list();
	}

	@Transactional(readOnly = true)
	public int getRowCountByCriteria(String name, String description,
			String title, String url_branding, String site_url,
			String admin_email, String notify_email, String notify_from,
			String notify_message, String workspace_type) {

		Criteria criteria = sessionFactory.getCurrentSession().createCriteria(
				Site.class);

		criteria.add(Restrictions.like("name", name + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("description", description + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("title", title + "", MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("url_branding", url_branding + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("site_url", site_url + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("admin_email", admin_email + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("notify_email", notify_email + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("notify_from", notify_from + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("notify_message", notify_message + "",
				MatchMode.ANYWHERE));
		criteria.add(Restrictions.like("workspace_type", workspace_type + "",
				MatchMode.ANYWHERE));

		criteria.setProjection(Projections.rowCount());

		return new Integer(criteria.uniqueResult() + "");
	}

}
