/*******************************************************************************
 * Copyright (c) 2011 BlueOxygen Technology.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the BlueOxygen Software License v1.0
 * which accompanies this distribution, and is available at
 *
 * Contributors:
 *     BlueOxygen Cimande Team - initial API and implementation
 *******************************************************************************/
package org.blueoxygen.cimande.social.dao;

import java.util.HashSet;
import java.util.Set;

import org.blueoxygen.cimande.persistence.hibernate.dao.HibernatePersistenceDaoManager;
import org.blueoxygen.cimande.security.User;
import org.blueoxygen.cimande.social.SocialAccount;
import org.hibernate.Query;
import org.hibernate.Session;
import org.springframework.stereotype.Repository;

/**
 * @author Dian Aditya
 * 
 */
@Repository
public class SocialAccountDao extends
		HibernatePersistenceDaoManager<SocialAccount> {

	public User getUserBySocialAccount(String provider, String username) {
		String hql = "FROM "
				+ SocialAccount.class.getName()
				+ " s WHERE s.socialProvider = :provider AND s.username = :username";

		Session session = sessionFactory.getCurrentSession();

		Query query = session.createQuery(hql);
		query.setParameter("provider", provider);
		query.setParameter("username", username);

		SocialAccount account = (SocialAccount) query.uniqueResult();

		return account == null ? null : account.getUser();
	}

	@SuppressWarnings("unchecked")
	public Set<SocialAccount> getSocialAccountByUser(String userId) {
		String hql = "FROM " + SocialAccount.class.getName()
				+ " s WHERE s.user.id = :userId";

		Query query = sessionFactory.getCurrentSession().createQuery(hql);
		query.setParameter("userId", userId);

		return new HashSet<SocialAccount>(query.list());
	}

	public User getUserByEmail(String email) {
		String hql = "FROM " + User.class.getName()
				+ " u WHERE u.email = :email";

		Query query = sessionFactory.getCurrentSession().createQuery(hql);
		query.setParameter("email", email);

		return (User) query.uniqueResult();
	}

	public void saveUser(User user) {
		sessionFactory.getCurrentSession().save(user);
	}
}
