/*******************************************************************************
 * Copyright (c) 2011 BlueOxygen Technology.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the BlueOxygen Software License v1.0
 * which accompanies this distribution, and is available at
 *
 * Contributors:
 *     BlueOxygen Cimande Team - initial API and implementation
 *******************************************************************************/
package org.blueoxygen.cimande.social.service;

import java.sql.Timestamp;
import java.util.Set;

import javax.inject.Inject;

import org.blueoxygen.cimande.commons.LogInformation;
import org.blueoxygen.cimande.persistence.GenericPersistenceManager;
import org.blueoxygen.cimande.role.Role;
import org.blueoxygen.cimande.security.User;
import org.blueoxygen.cimande.site.Site;
import org.blueoxygen.cimande.social.SocialAccount;
import org.blueoxygen.cimande.social.dao.SocialAccountDao;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author Dian Aditya
 * 
 */
@Service
@Transactional(readOnly = true)
public class SocialAccountServiceImpl implements SocialAccountService {

	@Inject
	private SocialAccountDao accountDao;

	@Inject
	private GenericPersistenceManager manager;

	public User getUserBySocialAccount(String provider, String username) {
		return accountDao.getUserBySocialAccount(provider, username);
	}

	@Transactional
	public User addNewUserBySocialAccount(SocialAccount socialAccount) {
		if (socialAccount == null)
			return null;

		// if (socialAccount.getUser() == null)
		// return null;

		LogInformation log = new LogInformation();
		log.setActiveFlag(1);
		log.setCreateDate(new Timestamp(System.currentTimeMillis()));
		log.setLastUpdateDate(new Timestamp(System.currentTimeMillis()));

		User user = socialAccount.getUser();
		if (user != null) {
			if (user.getId() == null) {
				user.setLogInformation(log);
			} else {
				user.getLogInformation().setLastUpdateBy(user.getId());
				user.getLogInformation().setLastUpdateDate(
						new Timestamp(System.currentTimeMillis()));
			}

			accountDao.saveUser(user);
		}

		socialAccount.setUser(user);
		socialAccount.setLogInformation(log);
		accountDao.persist(socialAccount);

		return socialAccount.getUser();
	}

	public User getUserByEmail(String email) {
		return accountDao.getUserByEmail(email);
	}

	public Set<SocialAccount> getSocialAccountByUser(String userId) {
		return accountDao.getSocialAccountByUser(userId);
	}

	public Role getDefaultRole() {
		return manager.getByUniqueField(Role.class, "default", "name");
	}

	public Site getDefaultSite() {
		return manager.getByUniqueField(Site.class, "default", "name");
	}
}
